/* OprofileDb - An Oprofile sample file database wrapper.
 * Written by Keith Seitz  <keiths@redhat.com>
 *
 * (c) 2003 Red Hat, Inc.
 *
 * This program is open source software licensed under the IBM
 * Common Public License ver. 1
 */
#ifndef _OPROFILEDB_H
#define _OPROFILEDB_H

#include <map>
#include <string>
#include <bfd.h>

#include "odb_hash.h"
#include "symbol.h"

class Sample;
class SymbolTable;

class OprofileDb
{
 public:
  // Creates an OprofileDb from the given sample file
  OprofileDb (const char *sample_file);
  ~OprofileDb ();

  // Function object used to compare VMA for sorting
  struct ltvma
  {
    bool operator() (const bfd_vma a, const bfd_vma b) const
    { return (a < b); }
  };

  // The type of the sample database returned be get_samples
  typedef std::map<const bfd_vma, Sample *, ltvma> samples_t;

  // The type of one sample in the database
  typedef std::pair<const bfd_vma, Sample *> sample_t;

  // Macro to fetch the Sample from the sample_t.
  static inline Sample *SAMPLE (sample_t sample) { return sample.second; }

  // Retrieves the sample database using STABLE as a symbol table (may be NULL)
  const samples_t &get_samples (SymbolTable *stable);

  // Retrieves the oprofile header of this sample file.
  const struct opd_header *get_header (void);

 protected:
  // Callback for walking oprofile sample database
  static void _get_samples_callback (odb_key_t key, odb_value_t info, void *data);

  // Makes sure the oprofile sample file is open
  void _open_db (void);

  // Closes the oprofile sample file
  void _close_db (void);

  // The sample file
  char *_filename;

  // The oprofile sample database for the file
  samples_odb_t *_tree;

  // A map of all the samples
  samples_t _samples;

  // The symbol table used to resolve VMA into symbols
  SymbolTable *_symbol_table;
};
#endif // !_OPROFILEDB_H
