/* Sample - A class which represents an Oprofile sample
 * Written by Keith Seitz  <keiths@redhat.com>
 *
 * (c) 2003 Red Hat, Inc.
 *
 * This program is open source software licensed under the IBM
 * Common Public License ver. 1
 */

#ifndef _SAMPLE_H
#define _SAMPLE_H
#include <stdlib.h>
#include <bfd.h>
#include "symbol.h"

class SymbolTable;

class Sample
{
 public:
  Sample (bfd_vma a, Symbol *symbol, unsigned int cnt);

  // Get the name of the symbol corresponding to this sample
  // Returns NULL if no symbol.
  inline const char *name (void) const
    { return (has_symbol () ? _symbol->name () : NULL); };

  // Returns the demangled name for this sample
  inline const char *demangled_name (void) const
    { return (has_symbol () ? _symbol->demangled_name () : NULL); };

  // Returns the total sample count for this sample
  inline unsigned int count(void) const { return _count; };

  // Increments the total sample count for this sample
  inline void incr_count (int n) { _count += n; };

  // Does this sample have a symbol?
  inline bool has_symbol (void) const { return _symbol != NULL; }

  // Returns the Symbol for this sample (could be NULL)
  inline Symbol *symbol (void) const { return _symbol; };

  // Gets the real vma for this sample
  inline bfd_vma vma (void) const { return _addr; }

 private:
  // (real) Address of sample
  bfd_vma _addr;

  // Symbol for sample (according to minimal symbols)
  Symbol *_symbol;

  // Number of times sample appears in output
  unsigned int _count;
};
#endif // !_SAMPLE_H
