/* SymbolTable - A symbol table class
 * Written by Keith Seitz  <keiths@redhat.com>
 *
 * (c) 2003 Red Hat, Inc.
 *
 * This program is open source software licensed under the IBM
 * Common Public License ver. 1
 */
#ifndef _STABLE_H
#define _STABLE_H
#include <bfd.h>
#include <vector>

class Symbol;
class Sample;

class SymbolTable
{
 public:
  SymbolTable (const char *file);
  ~SymbolTable (void);

  // Lookup a Symbol for the given sample vma. Returns
  // the symbol found (or NULL) and gives the real address
  // of the sample.
  Symbol *lookup_vma (bfd_vma sample_vma, bfd_vma &real_address);
  Symbol *lookup_vma (bfd_vma real_vma);

  // Read in the samples
  bool read_symbols (void);

  /* Gets the debug info for a given address:
     function name in debug info
     source filename
     line number

     Returns true if debug info found. False otherwise.
     NOTE: could return true but still have source_file and line be
     invalid!*/
  bool get_debug_info (bfd_vma vma, const char * &function,
		       const char * &source_file, unsigned int &line);

  // Debugging. Dump the symbol table
  void dump_table (void);

 protected:
  // Opens the BFD associated with the executable
  bool _open_bfd (void);

  // Closes the BFD
  void _close_bfd (void);

  // Helper function: is the given asymbol "interesting"? (i.e., should
  // it go into the symbol table?)
  static bool _interesting_symbol (asymbol *sym);

  // A list of known uninteresting symbols
  static char const *_boring_symbols[];

  // All of the executable's symbols
  std::vector<Symbol *> _symbols;

  // The executable's filename
  char *_filename;

  // The BFD associated with this executable
  bfd *_abfd;

  // The BFD symbol table
  asymbol **_symbol_table;

  // The physical load address of this executable (NOT THE BFD SECTION
  // START ADDRESS)
  bfd_vma _start_vma;

  // A performance cache
  Symbol *_cache_symbol;
};
#endif // !_STABLE_H
