/* Symbol - A class which represents symbols in executables
 * Written by Keith Seitz  <keiths@redhat.com>
 *
 * (c) 2003 Red Hat, Inc.
 *
 * This program is open source software licensed under the IBM
 * Common Public License ver. 1
 */
#include <stdlib.h>

#include "symbol.h"

// From libiberty
#ifndef DMGL_PARAMS
#define DMGL_PARAMS (1 << 0)	// Include function arguments
#endif

#ifndef DMGL_ANSI
#define DMGL_ANSI (1 << 1)	// Include const, volatile, etc
#endif

extern "C" char *cplus_demangle (char const *mangled_name, int options);

const bfd_vma Symbol::UNSET = (bfd_vma) -1;

Symbol::Symbol (asymbol *sym)
  : _end (UNSET), _asymbol (sym)
{
}

bool
Symbol::contains (bfd_vma addr) const
{
  if (addr >= start() && addr < _end)
    return true;
  return false;
}

const char *
Symbol::demangled_name (void) const
{
  char *demangled = cplus_demangle (name (), DMGL_PARAMS | DMGL_ANSI);
  if (demangled == NULL)
    return name ();

  return demangled;
}
