/* Symbol - A class which represents symbols in executables
 * Written by Keith Seitz  <keiths@redhat.com>
 *
 * (c) 2003 Red Hat, Inc.
 *
 * This program is open source software licensed under the IBM
 * Common Public License ver. 1
 */
#ifndef _SYMBOL_H
#define _SYMBOL_H

#include <bfd.h>

class Symbol
{
 public:
  Symbol (asymbol *sym);

  // Get the start address of this symbol
  inline bfd_vma start (void) const { return bfd_asymbol_value (_asymbol); };

  // Get the end address of this symbol
  inline bfd_vma end (void) const { return _end; };

  // Set the end address of this symbol
  inline void end (bfd_vma addr) { _end = addr; };

  // Get BFD's symbol info
  inline asymbol *get_asymbol (void) const { return _asymbol; };

  // Get BFD's section info
  inline asection *section (void) const { return bfd_get_section (_asymbol); };

  // Get the name of this symbol
  inline const char *name (void) const { return bfd_asymbol_name (_asymbol); };

  // Get the demangled name of this symbol (could be the same as name())
  const char *demangled_name (void) const;

  // Get the BFD flags for this symbol (i.e., BSF_FUNCTION, BSF_GLOBAL)
  inline flagword flags (void) const { return _asymbol->flags; };

  // Does this symbol contain the address ADDR?
  bool contains (bfd_vma addr) const;

  // Constant that all unset address are set to
  static const bfd_vma UNSET;

 protected:
  // The end address of this symbol (can only be set once all symbols read)
  bfd_vma _end;

  // The BFD symbol
  asymbol *_asymbol;
};
#endif // !_SYMBOL_H
